# Set global parameters here related to Cosmology (more to be added)
# Om= matter density Ol=dark energy density at the present epoch
# Planck 2015
# Om = 0.3121
# Ol = 0.6879
# WMAP9
# Om = 0.279
# Ol = 0.721
# Default
Om = 0.3
Ol = 0.7
